# coding=utf-8
from autosubmit.date.chunk_date_lib import chunk_start_date, chunk_end_date
from autosubmit.date.chunk_date_lib import parse_date


class ExperimentManager(object):
    """
    Encapsulates all chunk related tasks

    :param startdates: startdates to run
    :type startdates: list[str]
    :param members: members to run
    :type members: list[int]
    :param num_chunks: chunks to run
    :type num_chunks: int
    :param chunk_size: length of the chunks in months
    :type chunk_size: int
    :param member_digits: minimum number of digits to use in the member name
    :type member_digits: int
    :param calendar: experiment's calendar
    :type calendar: str
    """

    def __init__(self, startdates, members, num_chunks, chunk_size, member_digits, calendar='standard'):
        self.startdates = startdates
        self.members = members
        self.num_chunks = num_chunks
        self.chunk_size = chunk_size
        self.member_digits = member_digits
        self.calendar = calendar

    def get_chunk_list(self):
        """
        Return a list with all the chunks
        :return: List containing tuples of startdate, member and chunk
        :rtype: tuple[str, int, int]
        """
        chunk_list = list()
        for startdate in self.startdates:
            for member in self.members:
                for chunk in range(1, self.num_chunks + 1):
                    chunk_list.append((startdate, member, chunk))
        return chunk_list

    def get_member_list(self):
        """
        Return a list with all the members
        :return: List containing tuples of startdate and member
        :rtype: tuple[str, int, int]
        """
        member_list = list()
        for startdate in self.startdates:
            for member in self.members:
                    member_list.append((startdate, member))
        return member_list

    def get_year_chunks(self, startdate, year):
        """
        Get the list of chunks containing timesteps from the given year
        :param startdate: startdate to use
        :type startdate: str
        :param year: reference year
        :type year: int
        :return: list of chunks containing data from the given year
        :rtype: list[int]
        """
        date = parse_date(startdate)
        chunks = list()
        for chunk in range(1, self.num_chunks + 1):
            chunk_start = chunk_start_date(date, chunk, self.chunk_size, 'month', self.calendar)
            if chunk_start.year > year:
                break
            elif chunk_start.year == year or chunk_end_date(chunk_start, self.chunk_size, 'month',
                                                            self.calendar).year == year:
                chunks.append(chunk)

        return chunks

    def get_full_years(self, startdate):
        """
        Returns the list of full years that are in the given startdate
        :param startdate: startdate to use
        :type startdate: str
        :return: list of full years
        :rtype: list[int]
        """
        chunks_per_year = 12 / self.chunk_size
        date = parse_date(startdate)
        first_january = 0
        first_year = date.year
        if date.month != 1:
            month = date.month
            first_year += 1
            while month + self.chunk_size < 12:
                month += self.chunk_size
                first_january += 1

        years = list()
        for chunk in range(first_january, self.num_chunks - chunks_per_year, chunks_per_year):
            years.append(first_year)
            first_year += 1
        return years

    def get_member_str(self, member):
        """
        Returns the member name for a given member number.
        :param member: member's number
        :type member: int
        :return: member's name
        :rtype: str
        """
        return 'fc{0}'.format(str(member).zfill(self.member_digits))
