# coding=utf-8
from ConfigParser import SafeConfigParser
from autosubmit.config.log import Log
import re


# noinspection PyClassicStyleClass
class Parser(SafeConfigParser):
    """
    Class to manage the config file. It add options to manage default values and to convert strings to the
    desired types (int, bool, list ...)
    """

    def get_option(self, section, option, default=''):
        """
        Gets an option

        :param section: section that contains the option
        :type section: str
        :param option: option to get
        :type option: str
        :param default: value to be returned if option is not present
        :type default: object
        :return: option value
        :rtype: str
        """
        if self.has_option(section, option):
            return self.get(section, option)
        else:
            return default

    def get_list_option(self, section, option, default=list(), separator=' '):
        """
        Gets a list option

        :param section: section that contains the option
        :type section: str
        :param option: option to get
        :type option: str
        :param default: value to be returned if option is not present
        :type default: object
        :param separator: separator used to split the list
        :type separator: str
        :return: option value
        :rtype: list
        """
        if self.has_option(section, option):
            return self.get(section, option).split(separator)
        else:
            return default

    def get_bool_option(self, section, option, default=True):
        """
        Gets a boolean option

        :param section: section that contains the option
        :type section: str
        :param option: option to get
        :type option: str
        :param default: value to be returned if option is not present
        :type default: bool
        :return: option value
        :rtype: bool
        """
        if self.has_option(section, option):
            return self.get(section, option).lower().strip() == 'true'
        else:
            return default

    def get_int_option(self, section, option, default=0):
        """
        Gets an integer option

        :param section: section that contains the option
        :type section: str
        :param option: option to get
        :type option: str
        :param default: value to be returned if option is not present
        :type default: int
        :return: option value
        :rtype: int
        """
        if self.has_option(section, option):
            return int(self.get(section, option))
        else:
            return default

    def get_float_option(self, section, option, default=0.0):
        """
        Gets a float option

        :param section: section that contains the option
        :type section: str
        :param option: option to get
        :type option: str
        :param default: value to be returned if option is not present
        :type default: float
        :return: option value
        :rtype: float
        """
        if self.has_option(section, option):
            return float(self.get(section, option))
        else:
            return default

    def check_exists(self, section, option):
        """
        Checks if an option exists

        :param section: section that contains the option
        :type section: str
        :param option: option to check
        :type option: str
        :return: True if option exists, False otherwise
        :rtype: bool
        """
        if self.has_option(section, option):
            return True
        else:
            Log.error('Option {0} in section {1} not found'.format(option, section))
            return False

    def check_is_boolean(self, section, option, must_exist):
        """
        Checks if an option is a boolean value

        :param section: section that contains the option
        :type section: str
        :param option: option to check
        :type option: str
        :param must_exist: if True, option must exist
        :type must_exist: bool
        :return: True if option value is boolean, False otherwise
        :rtype: bool
        """
        if must_exist and not self.check_exists(section, option):
            Log.error('Option {0} in section {1} must exist'.format(option, section))
            return False
        if self.get_option(section, option, 'false').lower() not in ['false', 'true']:
            Log.error('Option {0} in section {1} must be true or false'.format(option, section))
            return False
        return True

    def check_is_choice(self, section, option, must_exist, choices):
        """
        Checks if an option is a valid choice in given self

        :param section: section that contains the option
        :type section: str
        :param option: option to check
        :type option: str
        :param must_exist: if True, option must exist
        :type must_exist: bool
        :param choices: valid choices
        :type choices: list
        :return: True if option value is a valid choice, False otherwise
        :rtype: bool
        """
        if must_exist and not self.check_exists(section, option):
            return False
        value = self.get_option(section, option, choices[0])
        if value not in choices:
            Log.error('Value {2} in option {0} in section {1} is not a valid choice'.format(option, section, value))
            return False
        return True

    def check_is_int(self, section, option, must_exist):
        """
        Checks if an option is an integer value

        :param section: section that contains the option
        :type section: str
        :param option: option to check
        :type option: str
        :param must_exist: if True, option must exist
        :type must_exist: bool
        :return: True if option value is integer, False otherwise
        :rtype: bool
        """
        if must_exist and not self.check_exists(section, option):
            return False
        value = self.get_option(section, option, '1')
        try:
            int(value)
        except ValueError:
            Log.error('Option {0} in section {1} is not valid an integer'.format(option, section))
            return False
        return True

    def check_regex(self, section, option, must_exist, regex):
        """
        Checks if an option complies with a regular expression

        :param section: section that contains the option
        :type section: str
        :param option: option to check
        :type option: str
        :param must_exist: if True, option must exist
        :type must_exist: bool
        :param regex: regular expression to check
        :type regex: str
        :return: True if option complies with regex, False otherwise
        :rtype: bool
        """
        if must_exist and not self.check_exists(section, option):
            return False
        prog = re.compile(regex)
        value = self.get_option(section, option, '1')
        if not prog.match(value):
            Log.error('Option {0} in section {1} is not valid: {2}'.format(option, section, value))
            return False
        return True

